# encoding: utf-8

require 'spec_helper'

describe Optimizer::Relation::Operation::Binary::MaterializedOperands, '#optimizable?' do
  subject { object.optimizable? }

  let(:materialized)     { Relation::Materialized.new([[:id, Integer]], [[1]])       }
  let(:not_materialized) { Relation.new([[:id, Integer]], LazyEnumerable.new([[1]])) }
  let(:left)             { not_materialized                                          }
  let(:right)            { not_materialized                                          }
  let(:relation)         { left.union(right)                                         }
  let(:object)           { described_class.new(relation)                             }

  context 'when the right and right are materialized' do
    let(:left)  { materialized }
    let(:right) { materialized }

    it { should be(true) }
  end

  context 'when the left is materialized' do
    let(:left) { materialized }

    it { should be(false) }
  end

  context 'when the right is materialized' do
    let(:right) { materialized }

    it { should be(false) }
  end

  context 'when the left and right are not materialized' do
    it { should be(false) }
  end
end
